/*  gngeo a neogeo emulator
 *  Copyright (C) 2001 Peponas Mathieu
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "../neocd.h"
#include "streams.h"
#include "sound.h"

int sound=1;
SDL_AudioSpec desired;

#define BUFFER_LEN 16384
extern Uint16 play_buffer[BUFFER_LEN]__attribute__((aligned(32)));
extern Uint16 left_buffer[BUFFER_LEN]__attribute__((aligned(32)));
extern Uint16 right_buffer[BUFFER_LEN]__attribute__((aligned(32)));
int *stream;
void update_sdl_stream(void *userdata, Uint8 * stream, int len)
{
    streamupdate(len);
    memcpy(stream, (Uint8 *) play_buffer, len);
		
}
static const uint8 stream_driver[] = {
	#include "aica_fw.h"
};

#define ARM_OFFSET 0xa0800000
static volatile uint32 *start  = (uint32*)(0x10000 + ARM_OFFSET);
static volatile uint32 *init_done = (uint32*)(0x10004 + ARM_OFFSET);
static volatile uint32 *play_start = (uint32*)(0x10008 + ARM_OFFSET);
static volatile uint32 *rate = (uint32*)(0x10010 + ARM_OFFSET);
static volatile uint32 *bits = (uint32*)(0x10014 + ARM_OFFSET);
static volatile uint32 *buf_len = (uint32*)(0x10018 + ARM_OFFSET);
static volatile uint32 *position  = (uint32*)(0x10020 + ARM_OFFSET);
#define BASE_ADDRESS 0x11000

static const uint32 base = 0xa0800000 + 0x11000;

void play_audio(void){

	streamupdate(16384);

	spu_memload( 0x11000, *play_buffer,16384);

	aica_play(0,0,0x11000,0,2048,22050,255,128,1);

}

int init_sdl_audio(void){
	  
		aica_init();
	spu_memset( 0x11000, 0,16384);
	
  *init_done = 0; /* need before spu_memload */

  g2_fifo_wait();
  
  spu_disable();
  spu_memload (0, &stream_driver, sizeof(stream_driver));
  spu_enable ();
  
  *rate = SAMPLE_RATE;
  *bits = 16;
  *buf_len = BUFFER_LEN * 2; /* DOUBLE BUFFERING */

  g2_fifo_wait ();
  
  for (;;)
  {
    if (*init_done) break;
    timer_spin_sleep (100);
  }
  
  *start = 1;
  g2_fifo_wait ();
  
  for (;;)
  {
    if (*play_start) break;
    timer_spin_sleep (100);
  }

    streams_sh_start();
	YM2610_sh_start();

	return 1;
}

int init_old_audio(void)
{

    if(SDL_InitSubSystem(SDL_INIT_AUDIO)) {
    	printf("Couldn't Start Sound.\n");
    }

    desired.freq = 22050;
    desired.samples = 1024;
    
    desired.format = AUDIO_S16;
    desired.channels = 1;
    desired.callback = update_sdl_stream;
    desired.userdata = NULL;
    int q = SDL_OpenAudio(&desired, NULL);
	printf("SDL_OpenAudio: %d",q);
    
    streams_sh_start();
	YM2610_sh_start();
	SDL_PauseAudio(0);
    
    return 1;
}

void sound_toggle(void) {
//	SDL_PauseAudio(sound);
	sound^=1;
}

void sound_shutdown(void) {
	SDL_PauseAudio(1);
    SDL_CloseAudio();
    SDL_QuitSubSystem(SDL_INIT_AUDIO);
    
    streams_sh_stop();
    YM2610_sh_stop();
    
}



